import asyncio
from pathlib import Path

import sqlalchemy as sa
from fin_data.sources.td_ameritrade.options.quotes_etl import TDOptionChains
from fin_data.sources.td_ameritrade.options.tables import td_option_payloads

payload_ids_file = Path(__file__) / "payload_ids.txt"
payload_ids = []
if payload_ids_file.is_file():
    with payload_ids_file.open(mode="r") as f:
        payload_ids = [r.strip() for r in f]


async def update_db():
    while True:
        try:
            engine = sa.create_engine(
                "postgresql://dank:JoeyRiley01@192.168.1.2:5653/danklabs"
            )
            inst = await TDOptionChains.create()
            with engine.connect() as conn:
                result = conn.execution_options(stream_results=True).execute(
                    sa.select(
                        td_option_payloads.c.payload_id, td_option_payloads.c.payload
                    )
                    .select_from(td_option_payloads)
                    .where(td_option_payloads.c.payload_id.not_in([payload_ids]))
                )
                count = 0
                for rows in result.partitions(50):
                    count += 1
                    print(f"****STARTING ROW BATCH {count}****")
                    for payload_id, row in rows:
                        print(payload_id)
                        inst.parse_response(row)
                        payload_ids.append(payload_id)

                    await inst._underlying_quotes_upserter.upsert()
                    await inst._quotes_upserter.upsert()
                    await inst._contracts_upserter.upsert()
                    await inst._payload_upserter.upsert()
                print("--------------SHUTTING DOWN---------------")
                await inst.shutdown()
                print("*************FINISHED***************")
                break
        except Exception as e:
            print(f"ERROR ({type(e)}): {e}")


if __name__ == "__main__":
    from dotenv import load_dotenv

    load_dotenv("/home/dan/.env")

    asyncio.run(update_db())
