"""This API returns the company information, financial ratios, and other key metrics for the equity specified.
Data is generally refreshed on the same day a company reports its latest earnings and financials.
"""

import os
from functools import cache, partial
from typing import List, Union

from fin_data import asset_info as ai
from database_utils.upsert_sources import SourcesUpserter
from ready_logger import get_logger

from .fetch import fetch

logger = get_logger(name="alphavantage_ipo")

# create the listing status loader.
UpserterT = partial(
    SourcesUpserter.create,
    source="alphavantage",
    logger=logger,
)


async def create_upserters():
    upserters = {
        e: await UpserterT.create(e)
        for e in (
            ai.Name,
            ai.Exchange,
            ai.Country,
            ai.Sector,
            ai.Industry,
            ai.FiscalYearEnd,
            ai.Industry,
            ai.FullTimeEmployees,
            ai.CIK,
            ai.Currency,
        )
    }
    upserters.update(
        {
            ai.Summary: await UpserterT.create(
                ai.Summary, name_map={"Description": "summary"}
            ),
            ai.BusinessAddress: await UpserterT.create(
                ai.BusinessAddress, name_map={"Address": "business_address"}
            ),
        }
    )


async def company_info(symbols: Union[str, List[str]]):
    if not isinstance(symbols, (list, tuple, set)):
        symbols = [symbols]
    for symbols in symbols:
        data = await fetch(
            f"https://www.alphavantage.co/query?function=OVERVIEW&symbol={symbol}&apikey={os.environ['ALPHAVANTAGE_API_KEY']}",
            json=True,
        )
        # TODO parse rows.
        row = {
            **data,
            "asset_id": data["Symbol"],
            "asset_id_type": ai.CompanyIDType.SYMBOL,
        }
