from database_utils.upsert_sources import SourcesUpserter
from fin_data import asset_info as ai
from fin_data.asset_info import CompanyIDType
from fin_data.utils import create_http_client, load_csv_with_header
from ready_logger import get_logger


async def load_options():
    source_urls = {
        "cboe_weeklys_options": "https://www.cboe.com/us/options/symboldir/weeklys_options/?download=csv",
        "cboe_equity_index_options": "https://www.cboe.com/us/options/symboldir/equity_index_options/?download=csv",
        "cboe_quarterly_options": "https://www.cboe.com/us/options/symboldir/quarterlys_options/?download=csv",
    }
    logger = get_logger(name="cboe_options")

    async with create_http_client() as client:
        for source, url in source_urls.items():
            resp = await client.get(url)
            logger.info(f"[{resp.status}] {url}")
            text = await resp.text()

            rows = load_csv_with_header(text)
            logger.info(f"Loaded {len(rows)} rows from CSV.")

            name_loader = await SourcesUpserter.create(
                sa_obj=ai.Name,
                source=source,
                logger=logger,
            )
            await name_loader.upsert(
                [
                    {
                        "name": row["Company Name"],
                        "name_type": ai.NameType.COMPANY,
                        "asset_id": row["Stock Symbol"],
                        "asset_id_type": CompanyIDType.SYMBOL,
                    }
                    for row in rows
                ]
            )


if __name__ == "__main__":
    import asyncio

    from dotenv import load_dotenv

    load_dotenv("/home/dan/danklabs/.localenv")

    asyncio.run(load_options())
