import requests
import sqlalchemy as sa
from sqlalchemy import func
from sqlalchemy.dialects.postgresql import ARRAY
from tqdm import tqdm

"""
class IEXCloudCompInfo(SaCompanyData):
    __table_args__ = {"schema": SCHEMA_NAME}
    __tablename__ = "iex_cloud"

    symbol = sa.Column(sa.String, primary_key=True)
    name = sa.Column(sa.String)
    exchange = sa.Column(sa.String)
    industry = sa.Column(sa.String)
    website = sa.Column(sa.String)
    summary = sa.Column(sa.String)
    ceo = sa.Column(sa.String)
    security_name = sa.Column(sa.String)
    sector = sa.Column(sa.String)
    sic = sa.Column(sa.String)
    employees = sa.Column(sa.Integer)
    tags = ARRAY(sa.String, dimensions=1)
    business_address = ARRAY(sa.String, dimensions=1)
    state = sa.Column(sa.String)
    city = sa.Column(sa.String)
    zip = sa.Column(sa.String)
    country = sa.Column(sa.String)
    phone = sa.Column(sa.String)
    logo = sa.Column(sa.String)
    created = sa.Column(sa.DateTime(timezone=True), default=func.now())
    updated = sa.Column(sa.DateTime(timezone=True), default=func.now())
"""


def get_json_dict(url: str):
    resp = requests.get(url)
    print(f"[{resp.status_code}] {url}")
    if resp.status_code == 200:
        return resp.json()
    return {}


def company_logo(symbol: str):
    # credits: 1 per call
    data = get_json_dict(
        f"https://cloud.iexapis.com/stable/stock/{symbol}/logo?token={IEXCLOUD_API_KEY}"
    )
    if data:
        return data.get("url")


def company_info(symbol: str):
    # credits: 1 per call.
    field_map = {
        "symbol": "symbol",
        "companyName": "name",
        "exchange": "exchange",
        "industry": "industry",
        "website": "website",
        "description": "summary",
        "CEO": "ceo",
        "securityName": "security_name",
        "sector": "sector",
        "primarySicCode": "sic",
        "employees": "employees",
        "tags": "tags",
        "state": "state",
        "city": "city",
        "zip": "zip",
        "country": "country",
        "phone": "phone",
    }

    data = get_json_dict(
        f"https://cloud.iexapis.com/stable/stock/{symbol}/company?token={IEXCLOUD_API_KEY}"
    )
    if data:
        data = {k: v for k, v in data.items() if v not in ("null", None)}
        address = []
        if "address" in data:
            address.append(data.pop("address"))
        if "address2" in data:
            address.append(data.pop("address2"))
        data = {field_map[k]: v for k, v in data if k in field_map}
        if len(address):
            data["business_address"] = address
        # download logo.
        logo = company_logo(symbol)
        if logo is not None:
            data["logo"] = logo
        return data


def symbol_json_to_database(conn, callback):
    table = PGTableWatch(
        conn=conn,
        table_name="iexcloud",
        schema_name="company_info",
        primary_key_col="symbol",
    )
    symbols = get_symbols_priority_order(conn, "company_info.iexcloud")
    for symbol in tqdm(symbols):
        data = callback(symbol)
        if data:
            if "symbol" not in data:
                data["symbol"] = symbol
            table.insert_or_update(data)
        sleep(1)


if __name__ == "__main__":
    conn = get_sql_conn()
    symbol_json_to_database(conn, company_info)
    conn.close()
