from utils import fetch, get_spider, get_sql_conn
from pprint import pformat
import aioredis
import asyncio
import argparse
import json

spider = redis_client = None


async def scrape_company_page(url):
    page = await fetch(spider, url, waitUntil=["load", "networkidle2"])
    data = {"url": url}
    name = await page.xpath('//h1[@class="title inline-block"]')
    if len(name):
        name = await page.evaluate("(ele) => ele.innerText", name[0])
        data["name"] = name.strip()
    overview = await page.xpath('//p[@class="muted-copy Sans-14px-black-60%"]')
    if len(overview):
        overview = await page.evaluate("(ele) => ele.innerText", overview[0])
        data["overview"] = overview.strip()
    employees = await page.xpath('//a[@data-control-name="topcard_employees"][@href]')
    if len(employees):
        data["employees_url"] = await page.evaluate(
            '(ele) => ele.getAttribute("href")', employees[0]
        )
        employee_count = await page.evaluate("(ele) => ele.innerText", employees[0])
        data["employee_count"] = employee_count.replace("employees", "").strip()
    decsision_makers = await page.xpath(
        '//a[@data-control-name="num_decision-makers"][@href]'
    )
    if len(decsision_makers):
        data["decsision_makers_url"] = await page.evaluate(
            '(ele) => ele.getAttribute("href")', decsision_makers[0]
        )
        decsision_maker_count = await page.evaluate(
            "(ele) => ele.innerText", decsision_makers[0]
        )
        data["decsision_maker_count"] = decsision_maker_count.replace(
            "decision makers", ""
        ).strip()
    description = await page.xpath(
        '//section[@class="entity-hovercard__description box-sizing-border-box full-width Sans-14px-black-75% ph4"]'
    )
    if len(description):
        description = await page.evaluate("(ele) => ele.innerText", description[0])
        data["description"] = description.strip()
    else:
        description = await page.xpath(
            """//code[@style="display: none"][@id][contains(text(),'"description":')]"""
        )
        if len(description):
            description = await page.evaluate("(ele) => ele.innerText", description[0])
            try:
                description = json.loads(description.strip())["description"]
            except Exception as e:
                print(f"ERROR LOADING DESCRIPTION FROM JSON: {e}")
            data["description"] = description.strip()
    website = await page.xpath('//a[@data-control-name="topcard_website"][@href]')
    if len(website):
        data["website"] = await page.evaluate(
            '(ele) => ele.getAttribute("href")', website[0]
        )
    headquarters_maps = await page.xpath(
        '//a[@data-control-name="topcard_headquarters"][@href]'
    )
    if len(headquarters_maps):
        data["headquarters_maps_url"] = await page.evaluate(
            '(ele) => ele.getAttribute("href")', headquarters_maps[0]
        )
    headquarters_addr = await page.xpath(
        '//div[@class="artdeco-hoverable-content__content"]'
    )
    if len(headquarters_addr) > 1:
        headquarters_addr = await page.evaluate(
            "(ele) => ele.innerText", headquarters_addr[1]
        )
        data["headquarters_addr_url"] = headquarters_addr.strip()
    # nav to insights page.
    await spider.set_idle(page)
    page = await fetch(spider, f"{url}/insights")
    department_stats = [
        await page.evaluate("(ele) => ele.innerText", ele)
        for ele in await page.xpath(
            '//section[@class="company-employee-insights-item"]'
        )
    ]
    if len(department_stats):
        data["department_stats"] = json.dumps([t.strip() for t in department_stats])
    while True:
        show_more_btn = await page.xpath(
            '//div[contains(@class,"show-more")][contains(@class,"button")]'
        )
        if not len(show_more_btn):
            break
        print("Loading more news articles.")
        await show_more_btn[0].click()
        await asyncio.sleep(1.5)
    news_articles = []
    for news_article in await page.xpath('//a[@class="insight-link"]'):
        title = await page.evaluate("(ele) => ele.innerText", news_article)
        news_articles.append(
            {
                "title": title.strip(),
                "url": await page.evaluate(
                    '(ele) => ele.getAttribute("href")', news_article
                ),
            }
        )
    if len(news_articles):
        data["news_articles"] = json.dumps(news_articles)
    await spider.set_idle(page)
    print(f"Finished company: {pformat(data)}")
    return data


def abs_url(url):
    if "linkedin.com" not in url:
        return f"https://www.linkedin.com/{url.lstrip('/')}"
    return url


async def scrape_company_pages(conn):
    while True:
        url = await redis_client.spop("linkedin_company_urls", encoding="utf-8")
        if url is None:
            logging.info("Finished all searches.")
            return
        if "?keywords" not in url:
            url = abs_url(url)
            data = await scrape_company_page(url)
            await asyncio.sleep(5000000000)
            with conn.cursor() as cur:
                cur.execute(
                    f"INSERT INTO linkedin_salesnav_companies ({','.join(list(data.keys()))}) VALUES ({','.join([f'%({k})s' for k in data.keys()])})",
                    data,
                )
            conn.commit()


async def queue_company_urls(conn):
    with conn.cursor() as cur:
        cur.execute(
            "SELECT DISTINCT company_url FROM linkedin_salesnav_search WHERE company_url IS NOT NULL"
        )
        urls = set(u[0] for u in cur.fetchall())
    print(f"Queueing {len(urls)} company profile urls")
    await redis_client.sadd("linkedin_company_urls", *urls)


def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument("-q", "--queue", help="Queue searches.", action="store_true")
    return parser.parse_args()


async def main():
    global redis_client, spider
    redis_client = await aioredis.create_redis("redis://127.0.0.1:6379/0")
    args = parse_args()
    conn = get_sql_conn()
    if args.queue:
        await queue_company_urls(conn)
    spider = await get_spider()
    page = await fetch(spider, "https://www.linkedin.com/uas/login")
    await asyncio.sleep(3)
    await spider.set_idle(page)
    # nav to homepace.
    _, page = await spider.get(
        "https://www.linkedin.com/sales/homepage", waitUntil=["load", "networkidle2"]
    )
    await asyncio.sleep(3)
    await spider.set_idle(page)
    await scrape_company_pages(conn)
    conn.close()
    redis_client.close()
    await spider.shutdown()


if __name__ == "__main__":
    asyncio.run(main())
