import csv
import re
from contextlib import contextmanager
from datetime import datetime
from ftplib import FTP
from pathlib import Path
from tempfile import NamedTemporaryFile
from typing import Dict, List

from ready_logger import logger


def load_rows(file_path: Path) -> List[Dict[str, str]]:
    with file_path.open(mode="r") as f:
        header = next(csv.reader(f, delimiter="|"))
        # load rows and remove footer.
        rows = list(csv.DictReader(f, fieldnames=header, delimiter="|"))
    # Get file timestamp. First cell of last row is update time in mmddyyyyhhmm format and all other cells have no data.
    timestamp = rows.pop()[header[0]]
    timestamp = re.findall(
        "(?i)File Creation Time:\s*(\d{2})(\d{2})(\d{4})(\d{2}):(\d{2})", timestamp
    )
    assert len(timestamp) == 1
    timestamp = [int(t) for t in timestamp[0]]
    timestamp = dict(zip(["month", "day", "year", "hour", "minute"], timestamp))
    timestamp = datetime(**timestamp)
    for row in rows:
        row["updated"] = timestamp
    return rows, timestamp


@contextmanager
def downloaded_file(file_name: str) -> Path:
    logger.info(f"Downloading {file_name}")
    save_path = Path(NamedTemporaryFile().name)
    with FTP("ftp.nasdaqtrader.com") as ftp:
        ftp.login()
        ftp.cwd("SymbolDirectory")
        with save_path.open(mode="wb") as o:
            ftp.retrbinary(f"RETR {file_name}", o.write)
    yield save_path
    save_path.unlink()
