from time import sleep

import requests
import sqlalchemy as sa
from sqlalchemy import func
from sqlalchemy.dialects.postgresql import ARRAY
from tqdm import tqdm

"""
class PolygonCompInfo(SaCompanyData):
    __table_args__ = {"schema": SCHEMA_NAME}
    __tablename__ = "polygon"

    symbol = sa.Column(sa.String, primary_key=True)
    logo = sa.Column(sa.String)
    list_date = sa.Column(sa.String)
    cik = sa.Column(sa.String)
    sic = sa.Column(sa.String)
    industry = sa.Column(sa.String)
    sector = sa.Column(sa.String)
    employees = sa.Column(sa.String)
    phone = sa.Column(sa.String)
    ceo = sa.Column(sa.String)
    website = sa.Column(sa.String)
    summary = sa.Column(sa.String)
    exchange = sa.Column(sa.String)
    name = sa.Column(sa.String)
    exchange_symbol = sa.Column(sa.String)
    business_address = sa.Column(sa.String)
    state = sa.Column(sa.String)
    active = sa.Column(sa.String)
    tags = ARRAY(sa.String, dimensions=1)
    similar = ARRAY(sa.String, dimensions=1)
    country = ARRAY(sa.String, dimensions=1)
    created = sa.Column(sa.DateTime(timezone=True), default=func.now())

    updated = sa.Column(sa.DateTime(timezone=True), default=func.now())
"""


def get_json_dict(url: str):
    # 5 API call/min
    resp = requests.get(url)
    print(f"[{resp.status_code}] {url}")
    if resp.status_code == 200:
        return resp.json()
    if resp.status_code == 429:
        sleep(20)
        return get_json_dict(url)
    return {}


def company_info(symbol: str):
    field_map = {
        "logo": "logo",
        "listdate": "list_date",
        "cik": "cik",
        "sic": "sic",
        "country": "country",
        "industry": "industry",
        "sector": "sector",
        "employees": "employees",
        "phone": "phone",
        "ceo": "ceo",
        "url": "website",
        "description": "summary",
        "exchange": "exchange",
        "name": "name",
        "symbol": "symbol",
        "exchangeSymbol": "exchange_symbol",
        "hq_address": "business_address",
        "hq_state": "state",
        "similar": "similar",
        "active": "active",
        "tags": "tags",
    }
    data = get_json_dict(
        f"https://api.polygon.io/v1/meta/symbols/{symbol}/company?&apiKey={API_KEY}"
    )
    data = {field_map[k]: v for k, v in data.items() if k in field_map}
    if "symbol" not in data:
        data["symbol"] = symbol
    if "country" in data and not isinstance(data["country"], list):
        data["country"] = [data["country"]]
    return data


def json_to_database(conn, callback):
    table = PGTableWatch(
        conn=conn,
        table_name="polygon",
        schema_name="company_info",
        primary_key_col="symbol",
    )
    symbols = get_symbols_priority_order(conn, "company_info.polygon")
    for symbol in tqdm(symbols):
        data = callback(symbol)
        if data:
            table.insert_or_update(data)
        sleep(60 / 4.8)


def fundamentals(symbol: str, report_type: str = "Y"):
    # report_type: Y = Year YA = Year annualized Q = Quarter QA = Quarter Annualized T = Trailing twelve months TA = trailing twelve months annualized
    return get_mapped_json_dict(
        f"https://api.polygon.io/v2/reference/financials/{symbol}?limit=5&type={report_type}&apiKey={API_KEY}"
    )


def news(symbol: str, page: int = 1):
    return get_json_dict(
        f"https://api.polygon.io/v1/meta/symbols/{symbol}/news?perpage=50&page=1&apiKey={API_KEY}"
    )


if __name__ == "__main__":
    conn = get_sql_conn()
    json_to_database(conn, company_info)
    conn.close()
