import asyncio
from io import BytesIO
from pathlib import Path
from zipfile import ZipFile

import requests
from lxml.html import fromstring


async def get_zip_contents(url: str):
    resp = requests.get(url)
    print(f"[{resp.status}] {url}")
    files_dir = Path(url.split("/")[-1].split(".")[0])
    files_dir.mkdir()
    data = await resp.read()
    with ZipFile(BytesIO(data)) as zf:
        zf.extractall(path=str(files_dir))


async def get_all_files():
    resp = requests.get(
        "https://www.sec.gov/dera/data/financial-statement-and-notes-data-set.html",
    )
    html = resp.text
    root = fromstring(html)
    zip_urls = set(
        f"https://www.sec.gov{e}"
        for e in root.xpath("//table[@class='list']//a[contains(@href,'.zip')]/@href")
    )
    for url in zip_urls:
        await get_zip_contents(url)


async def main():
    asyncio.run(get_all_files())


if __name__ == "__main__":
    main()
