from typing import Any, Dict, List

from feedparser.util import FeedParserDict


async def parse_standard_feed(feed: FeedParserDict) -> List[Dict[str, Any]]:
    "Extract data from a standard SEC RSS feed. Works for everything but latest feed."
    entries = []
    for entry in feed.entries:
        filing_data = {
            "documents_url": entry["link"],
            "type": entry.get("edgar_formtype", entry.get("summary")).strip(),
            "name": entry["edgar_companyname"],
            "cik": entry["edgar_ciknumber"],
        }
        if "edgar_filingdate" in entry:
            month, day, year = entry["edgar_filingdate"].split("/")
            filing_data["filing_date"] = f"{year}-{month}-{day}"
        entries.append(filing_data)
    return entries
