from datetime import datetime
from enum import Enum, auto

import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

metadata = sa.MetaData(schema="sec")

no_matching_cik_table = sa.Table(
    "no_matching_cik",
    metadata,
    sa.Column("cik", sa.String, primary_key=True),
    sa.Column("time", sa.DateTime(timezone=False), default=datetime.utcnow),
)

no_matching_symbol_table = sa.Table(
    "no_matching_symbol",
    metadata,
    sa.Column("symbol", sa.String, primary_key=True),
    sa.Column("time", sa.DateTime(timezone=False), default=datetime.utcnow),
)


last_filing_searches = sa.Table(
    "last_filing_searches",
    metadata,
    sa.Column("cik", sa.String, primary_key=True),
    sa.Column("type", sa.String, primary_key=True),
    sa.Column("last_search", sa.DateTime(timezone=False), nullable=False),
)


class PageType(Enum):
    FILING = auto()
    SEARCH_RESULTS = auto()


failed_page_loads_table = sa.Table(
    "failed_page_loads",
    metadata,
    sa.Column("url", sa.String, primary_key=True),
    sa.Column("cik", sa.String),
    sa.Column("type", sa.String),
    sa.Column("page_type", postgresql.ENUM(PageType, schema=metadata.schema)),
    sa.Column(
        "last_fail",
        sa.DateTime(timezone=False),
        default=datetime.utcnow,
        nullable=False,
    ),
    sa.Column("times_failed", sa.Integer, default=1, nullable=False),
)

filings_table = sa.Table(
    "filings",
    metadata,
    sa.Column("data_id", sa.String, primary_key=True),
    sa.Column("cik", sa.String, index=True),
    sa.Column("type", sa.String, index=True),
    sa.Column("filing_date", sa.String),
    sa.Column("accepted_date", sa.String),
    sa.Column("file_film_num", sa.String),
    sa.Column("period_of_report", sa.String),
    sa.Column("sec_accession_num", sa.String),
    sa.Column("htm_url", sa.String),
    sa.Column("htm_url_score", sa.Float),
    sa.Column("txt_url", sa.String, index=True),
    sa.Column("txt_url_score", sa.Float),
    sa.Column("interactive_url", sa.String),
    sa.Column("documents_url", sa.String),
    sa.Column("file_name", sa.String),
    sa.Column(
        "created",
        sa.DateTime(timezone=False),
        default=datetime.utcnow,
        comment="DataIDIgnore",
    ),
)

company_search_table = sa.Table(
    "company_search",
    metadata,
    sa.Column("data_id", sa.String, primary_key=True),
    sa.Column("cik", sa.String, index=True),
    sa.Column("symbol", sa.String, index=True),
    sa.Column("name", sa.String),
    sa.Column("formerly", sa.String),
    sa.Column("state", sa.String),
    sa.Column("state_of_inc", sa.String),
    sa.Column("mailing_address", sa.String),
    sa.Column("bussiness_address", sa.String),
    sa.Column("phone", sa.String),
    sa.Column("office", sa.String),
    sa.Column("fiscal_year_end", sa.String),
    sa.Column("sic", sa.String),
    sa.Column(
        "created",
        sa.DateTime(timezone=False),
        default=datetime.utcnow,
        comment="DataIDIgnore",
    ),
    sa.Column(
        "updated",
        sa.DateTime(timezone=False),
        default=datetime.utcnow,
        comment="DataIDIgnore",
    ),
)

daily_rss_table = sa.Table(
    "daily_rss",
    metadata,
    sa.Column("cik", sa.String),
    sa.Column("name", sa.String),
    sa.Column("type", sa.String),
    sa.Column("filing_date", sa.DateTime(timezone=False)),
    sa.Column("documents_url", sa.String, primary_key=True),
)


rss_last_update_table = sa.Table(
    "rss_last_update",
    metadata,
    sa.Column("rss_url", sa.String, primary_key=True),
    sa.Column("updated", sa.DateTime(timezone=False), nullable=False),
)

quarterly_archive_table = sa.Table(
    "quarterly_archive",
    metadata,
    sa.Column("cik", sa.String),
    sa.Column("name", sa.String),
    sa.Column("type", sa.String),
    sa.Column("filing_date", sa.DateTime(timezone=False)),
    sa.Column("htm_url", sa.String),
    sa.Column("txt_url", sa.String),
)


daily_archive_table = sa.Table(
    "daily_archive",
    metadata,
    sa.Column("cik", sa.String),
    sa.Column("name", sa.String),
    sa.Column("type", sa.String),
    sa.Column("filing_date", sa.DateTime(timezone=False)),
    sa.Column("htm_url", sa.String),
    sa.Column("txt_url", sa.String),
)
