import sqlalchemy as sa

metadata = sa.MetaData(schema="td_ameritrade")

blacklist = sa.Table(
    "blacklist",
    metadata,
    sa.Column("endpoint", sa.String, primary_key=True),
    sa.Column("symbol", sa.String, primary_key=True),
)

td_mutual_funds = sa.Table(
    "mutual_funds",
    metadata,
    sa.Column("symbol", sa.String, primary_key=True, index=True),
    sa.Column("description", sa.String),
    sa.Column("close_price", sa.Float),
    sa.Column("net_change", sa.Float),
    sa.Column("total_volume", sa.Integer),
    sa.Column("trade_time", sa.DateTime(timezone=False)),
    sa.Column("exchange", sa.String),
    sa.Column("exchange_name", sa.String),
    sa.Column("digits", sa.Float),
    sa.Column("_52_wk_high", sa.Float),
    sa.Column("_52_wk_low", sa.Float),
    sa.Column("n_a_v", sa.Float),
    sa.Column("pe_ratio", sa.Float),
    sa.Column("div_amount", sa.Float),
    sa.Column("div_yield", sa.Float),
    sa.Column("div_date", sa.String),
    sa.Column("security_status", sa.String),
)


td_futures = sa.Table(
    "futures",
    metadata,
    sa.Column("symbol", sa.String, primary_key=True, index=True),
    sa.Column("bid_price", sa.Float),
    sa.Column("ask_price", sa.Float),
    sa.Column("last_price", sa.Float),
    sa.Column("bid_id", sa.String),
    sa.Column("ask_id", sa.String),
    sa.Column("high_price", sa.Float),
    sa.Column("low_price", sa.Float),
    sa.Column("close_price", sa.Float),
    sa.Column("exchange", sa.String),
    sa.Column("description", sa.String),
    sa.Column("last_id", sa.String),
    sa.Column("open_price", sa.Float),
    sa.Column("change", sa.Float),
    sa.Column("future_percent_change", sa.Float),
    sa.Column("exchange_name", sa.String),
    sa.Column("security_status", sa.String),
    sa.Column("open_interest", sa.Float),
    sa.Column("mark", sa.Float),
    sa.Column("tick", sa.Float),
    sa.Column("tick_amount", sa.Float),
    sa.Column("product", sa.String),
    sa.Column("future_price_format", sa.String),
    sa.Column("future_trading_hours", sa.String),
    sa.Column("future_is_tradable", sa.Float),
    sa.Column("future_multiplier", sa.Float),
    sa.Column("future_is_active", sa.Float),
    sa.Column("future_settlement_price", sa.Float),
    sa.Column("future_active_symbol", sa.String, primary_key=True, index=True),
    sa.Column("future_expiration_date", sa.String),
)


td_future_options = sa.Table(
    "future_options",
    metadata,
    sa.Column("symbol", sa.String, primary_key=True, index=True),
    sa.Column("bid_price", sa.Float),
    sa.Column("ask_price", sa.Float),
    sa.Column("last_price", sa.Float),
    sa.Column("high_price", sa.Float),
    sa.Column("low_price", sa.Float),
    sa.Column("close_price", sa.Float),
    sa.Column("description", sa.String),
    sa.Column("open_price", sa.Float),
    sa.Column("net_change", sa.Float),
    sa.Column("open_interest", sa.Float),
    sa.Column("exchange_name", sa.String),
    sa.Column("security_status", sa.String),
    sa.Column("volatility", sa.Float),
    sa.Column("money_intrinsic_value", sa.Float),
    sa.Column("multiplier", sa.Float),
    sa.Column("digits", sa.Float),
    sa.Column("strike_price", sa.Float),
    sa.Column("contract_type", sa.String),
    sa.Column("underlying", sa.String),
    sa.Column("time_value", sa.Float),
    sa.Column("delta", sa.Float),
    sa.Column("gamma", sa.Float),
    sa.Column("theta", sa.Float),
    sa.Column("vega", sa.Float),
    sa.Column("rho", sa.Float),
    sa.Column("mark", sa.Float),
    sa.Column("tick", sa.Float),
    sa.Column("tick_amount", sa.Float),
    sa.Column("future_is_tradable", sa.Float),
    sa.Column("future_trading_hours", sa.String),
    sa.Column("future_percent_change", sa.Float),
    sa.Column("future_is_active", sa.Float),
    sa.Column("future_expiration_date", sa.Float),
    sa.Column("expiration_type", sa.String),
    sa.Column("exercise_type", sa.String),
    sa.Column("in_the_money", sa.Float),
)


td_indexes = sa.Table(
    "indexes",
    metadata,
    sa.Column("symbol", sa.String, primary_key=True, index=True),
    sa.Column("description", sa.String),
    sa.Column("last_price", sa.Float),
    sa.Column("open_price", sa.Float),
    sa.Column("high_price", sa.Float),
    sa.Column("low_price", sa.Float),
    sa.Column("close_price", sa.Float),
    sa.Column("net_change", sa.Float),
    sa.Column("total_volume", sa.Integer),
    sa.Column("trade_time", sa.DateTime(timezone=False)),
    sa.Column("exchange", sa.String),
    sa.Column("exchange_name", sa.String),
    sa.Column("digits", sa.Float),
    sa.Column("_52_wk_high", sa.Float),
    sa.Column("_52_wk_low", sa.Float),
    sa.Column("security_status", sa.String),
)


td_forex = sa.Table(
    "forex",
    metadata,
    sa.Column("symbol", sa.String, primary_key=True, index=True),
    sa.Column("bid_price", sa.Float),
    sa.Column("ask_price", sa.Float),
    sa.Column("last_price", sa.Float),
    sa.Column("high_price", sa.Float),
    sa.Column("low_price", sa.Float),
    sa.Column("close_price", sa.Float),
    sa.Column("exchange", sa.String),
    sa.Column("description", sa.String),
    sa.Column("open_price", sa.Float),
    sa.Column("change", sa.Float),
    sa.Column("percent_change", sa.Float),
    sa.Column("exchange_name", sa.String),
    sa.Column("digits", sa.Float),
    sa.Column("security_status", sa.String),
    sa.Column("tick", sa.Float),
    sa.Column("tick_amount", sa.Float),
    sa.Column("product", sa.String),
    sa.Column("trading_hours", sa.String),
    sa.Column("is_tradable", sa.Float),
    sa.Column("market_maker", sa.String),
    sa.Column("_52_wk_high", sa.Float),
    sa.Column("_52_wk_low", sa.Float),
    sa.Column("mark", sa.Float),
)

td_etfs = sa.Table(
    "etfs",
    metadata,
    sa.Column("symbol", sa.String, primary_key=True, index=True),
    sa.Column("quote_time", sa.DateTime(timezone=False), primary_key=True),
    sa.Column("trade_time", sa.DateTime(timezone=False)),
    sa.Column("description", sa.String),
    sa.Column("bid_price", sa.Float),
    sa.Column("bid_size", sa.Integer),
    sa.Column("bid_id", sa.String),
    sa.Column("ask_price", sa.Float),
    sa.Column("ask_size", sa.Integer),
    sa.Column("ask_id", sa.String),
    sa.Column("last_price", sa.Float),
    sa.Column("last_size", sa.Integer),
    sa.Column("last_id", sa.String),
    sa.Column("open_price", sa.Float),
    sa.Column("high_price", sa.Float),
    sa.Column("low_price", sa.Float),
    sa.Column("close_price", sa.Float),
    sa.Column("net_change", sa.Float),
    sa.Column("total_volume", sa.Integer),
    sa.Column("mark", sa.Float),
    sa.Column("exchange", sa.String),
    sa.Column("exchange_name", sa.String),
    sa.Column("marginable", sa.Float),
    sa.Column("shortable", sa.Float),
    sa.Column("volatility", sa.Float),
    sa.Column("digits", sa.Float),
    sa.Column("_52_wk_high", sa.Float),
    sa.Column("_52_wk_low", sa.Float),
    sa.Column("pe_ratio", sa.Float),
    sa.Column("div_amount", sa.Float),
    sa.Column("div_yield", sa.Float),
    sa.Column("div_date", sa.String),
    sa.Column("security_status", sa.String),
    sa.Column("regular_market_last_price", sa.Float),
    sa.Column("regular_market_last_size", sa.Integer),
    sa.Column("regular_market_net_change", sa.Float),
    sa.Column("regular_market_trade_time", sa.DateTime(timezone=False)),
)


td_equities = sa.Table(
    "equities",
    metadata,
    sa.Column("symbol", sa.String, primary_key=True, index=True),
    sa.Column("quote_time", sa.DateTime(timezone=False), primary_key=True),
    sa.Column("trade_time", sa.DateTime(timezone=False)),
    sa.Column("description", sa.String),
    sa.Column("bid_price", sa.Float),
    sa.Column("bid_size", sa.Integer),
    sa.Column("bid_id", sa.String),
    sa.Column("ask_price", sa.Float),
    sa.Column("ask_size", sa.Integer),
    sa.Column("ask_id", sa.String),
    sa.Column("last_price", sa.Float),
    sa.Column("last_size", sa.Integer),
    sa.Column("last_id", sa.String),
    sa.Column("open_price", sa.Float),
    sa.Column("high_price", sa.Float),
    sa.Column("low_price", sa.Float),
    sa.Column("close_price", sa.Float),
    sa.Column("net_change", sa.Float),
    sa.Column("total_volume", sa.Integer),
    sa.Column("mark", sa.Float),
    sa.Column("exchange", sa.String),
    sa.Column("exchange_name", sa.String),
    sa.Column("marginable", sa.Float),
    sa.Column("shortable", sa.Float),
    sa.Column("volatility", sa.Float),
    sa.Column("digits", sa.Float),
    sa.Column("_52_wk_high", sa.Float),
    sa.Column("_52_wk_low", sa.Float),
    sa.Column("pe_ratio", sa.Float),
    sa.Column("div_amount", sa.Float),
    sa.Column("div_yield", sa.Float),
    sa.Column("div_date", sa.String),
    sa.Column("security_status", sa.String),
    sa.Column("regular_market_last_price", sa.Float),
    sa.Column("regular_market_last_size", sa.Integer),
    sa.Column("regular_market_net_change", sa.Float),
    sa.Column("regular_market_trade_time", sa.DateTime(timezone=False)),
)
