from collections import defaultdict

from database_utils.upsert import Upserter
from fin_data.sources.td_ameritrade.tables import blacklist
from ready_logger import get_logger


class BlackLister:
    logger = get_logger('BlackLister')

    @classmethod
    async def create(cls, max_bad_count: int = 4, max_bad_ratio: float = 0.7) -> None:
        self = cls()
        self.max_bad_count = max_bad_count
        self.max_bad_ratio = max_bad_ratio
        self._total = defaultdict(int)
        self._bad = defaultdict(int)
        self._upserter = await Upserter.create(sa_obj=blacklist, on_dupe_update=False)
        return self

    def mark_ok(self, endpoint: str, symbol: str):
        self._total[(endpoint, symbol)] += 1

    async def mark_bad(self, endpoint: str, symbol: str) -> True:
        key = (endpoint, symbol)
        self._bad[key] += 1
        self._total[key] += 1
        if (bad_count := self._bad[key]) >= self.max_bad_count and bad_count / self._total[key] >= self.max_bad_ratio:
            self.logger.warning(f"Blacklisting symbol: {symbol}, endpoint: {endpoint}")
            await self._upserter.upsert([{'endpoint': endpoint, 'symbol': symbol}])
            return True
        return False
            