from datetime import datetime, timedelta
from pathlib import Path

import numpy as np
import pandas as pd
import requests_cache
import yfinance as yf
from fin_data.utils import random_user
from ready_logger import logger
from tqdm import tqdm


async def download_data(
    symbol: str, td_days: int = 59, td_hours: int = 23
) -> pd.DataFrame:
    session = requests_cache.CachedSession("yfinance.cache")
    session.headers["User-Agent"] = random_user()
    end = datetime.now()
    start = end - timedelta(days=td_days, hours=td_hours)
    logger.info(f"Downloading {symbol}\t{start} - {end}")
    data = yf.Ticker(symbol, session=session).history(
        interval="2m", end=end, start=start
    )
    if not data.empty:
        data.reset_index(inplace=True)
        data.replace({np.nan: None}, inplace=True)
        data["symbol"] = symbol
        logger.info(f"Loading {data.shape} {symbol}")

        await loader.extend(data.to_dict("records"))
        splits = data[data["Stock Splits"] > 0]

        dividends = data[data["Dividends"] > 0]


async def load_from_files():
    import numpy as np
    from tqdm import tqdm

    """
    start_time = data.index[0].strftime("%Y%m%d")
    end_time = data.index[-1].strftime("%Y%m%d")
    file_name = f"{start_time}_{end_time}_{symbol}.parquet"
    """
    files = list(Path("/home/dan/yf_data").iterdir())
    for f in tqdm(files):
        df = pd.read_parquet(f)
        symbol = f.stem.split("_")[-1]
        df["Symbol"] = symbol
        df = df.reset_index().replace({np.nan: None})
        rows = df.to_dict("records")
        await loader.extend(rows)
    await loader.load()


async def download_all():
    engine = sa.create_engine(DB_URL)
    symbols = set(engine.execute(stock_symbols_querey).scalars())
    for s in tqdm(symbols):
        await download_data(s)
        await asyncio.sleep(1)
    await loader.load()


if __name__ == "__main__":
    import asyncio

    asyncio.run(download_all())
