import csv
import random
from io import StringIO
from typing import Dict, List, Union

import sqlalchemy as sa
from aiohttp import ClientSession, ClientTimeout, TCPConnector
from database_utils import to_table
from ready_logger import logger
from sqlalchemy.orm.decl_api import DeclarativeMeta
from sqlalchemy.sql.elements import quoted_name

USER_AGENTS = [
    "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/94.0.4606.54 Safari/537.36",
    "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:92.0) Gecko/20100101 Firefox/92.0",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/97.0.4692.71 Safari/537.36",
]


def create_http_client():
    return ClientSession(
        timeout=ClientTimeout(60_000), connector=TCPConnector(ssl=False)
    )


def load_csv_with_header(file_text: str) -> List[Dict[str, str]]:
    fo = StringIO(file_text)
    header = [c.strip() for c in next(csv.reader(fo))]
    return list(csv.DictReader(fo, fieldnames=header))


def random_user() -> str:
    return random.choice(USER_AGENTS)


def user_header() -> Dict[str, str]:
    return {"User-Agent": "Danklabs.org. Open Source Data."}  # random_user()}


def extract_column_name(sa_obj: Union[sa.Table, DeclarativeMeta]) -> quoted_name:
    table = to_table(sa_obj)
    column_names = table.columns.keys()
    if len(column_names) > 1:
        raise RuntimeError(
            "Entity %s has more than one column. (%s columns)",
            sa_obj,
            len(column_names),
        )
    return column_names[0]
